/*
 *  Kaidan - A user-friendly XMPP client for every device!
 *
 *  Copyright (C) 2016-2020 Kaidan developers and contributors
 *  (see the LICENSE file for a full list of copyright authors)
 *
 *  Kaidan is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  In addition, as a special exception, the author of Kaidan gives
 *  permission to link the code of its release with the OpenSSL
 *  project's "OpenSSL" library (or with modified versions of it that
 *  use the same license as the "OpenSSL" library), and distribute the
 *  linked executables. You must obey the GNU General Public License in
 *  all respects for all of the code used other than "OpenSSL". If you
 *  modify this file, you may extend this exception to your version of
 *  the file, but you are not obligated to do so.  If you do not wish to
 *  do so, delete this exception statement from your version.
 *
 *  Kaidan is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef REGISTRATIONDATAFORMMODEL_H
#define REGISTRATIONDATAFORMMODEL_H

#include "DataFormModel.h"
// Qt
#include <QList>
class QString;

/**
 * This class is used to store the data of the registration form.
 */
class RegistrationDataFormModel : public DataFormModel
{
	Q_OBJECT

public:
	using DataFormModel::DataFormModel;

	RegistrationDataFormModel(QObject *parent = nullptr);
	RegistrationDataFormModel(const QXmppDataForm &dataForm, QObject *parent = nullptr);

	Q_INVOKABLE bool hasUsernameField() const;
	Q_INVOKABLE bool hasPasswordField() const;
	Q_INVOKABLE bool hasEmailField() const;

	Q_INVOKABLE void setUsername(const QString &username);
	Q_INVOKABLE void setPassword(const QString &password);
	Q_INVOKABLE void setEmail(const QString &email);

	int usernameFieldIndex() const;
	int passwordFieldIndex() const;
	int emailFieldIndex() const;

	QXmppDataForm::Field extractUsernameField() const;
	QXmppDataForm::Field extractPasswordField() const;
	QXmppDataForm::Field extractEmailField() const;

	QString extractUsername() const;
	QString extractPassword() const;
	QString extractEmail() const;

	bool isFakeForm() const;
	void setIsFakeForm(bool isFakeForm);

	QVector<int> indiciesToFilter() const;

private:
	void initializeFilteredDataFormFields();

	bool m_isFakeForm = false;
	QList<QString> filteredDataFormFields;
};

#endif // REGISTRATIONDATAFORMMODEL_H
